"""Convert a series."""


def to_settlement_period(extension_array, series_of_strings):
    """Convert from a series from strings to settlement period objects."""
    return extension_array(series_of_strings)
